local gr, down = love.graphics, love.keyboard.isDown
local pi,sin,cos,sqrt = math.pi, math.sin, math.cos, math.sqrt

function love.load()
	sides	= 8
	sz		= 100
	sp		= 1
	center	= { 400, 300, 400 }
	rot		= {0, pi/8, pi/8 }
	createFigure( sides )
	timer	= 0
	showNormals = true
	text = {
		'Key Options',
		'rotation - arrows& jk',
		'movement - wasd',
		'size - nm',
		'normals = b',
		'# sides - cv'
		}
	end

function love.update( dt )
	dt = dt * sp
	rot[1]	= rot[1] + dt/1.5
	Ver		= {}
	for i	= 1, #ver do		-- rotate point cloud & convert to 2D space
		Ver[i]={d3TOd2(ver[i][1]*sz+center[1],ver[i][2]*sz+center[2],ver[i][3]*sz+center[3],rot,center)}
		end
	if down 'up' then
		rot[3] = rot[3] + dt
		elseif down 'down'	then rot[3] = rot[3] - dt
		elseif down 'left'	then rot[2] = rot[2] - dt
		elseif down 'right'	then rot[2] = rot[2] + dt
		elseif down 'j'		then rot[1] = rot[1] - dt
		elseif down 'k'		then rot[1] = rot[1] + dt
		elseif down 'd'		then center[1] = center[1] + 80 * dt
		elseif down 'a'		then center[1] = center[1] - 80 * dt
		elseif down 'w'		then center[2] = center[2] - 80 * dt
		elseif down 's'		then center[2] = center[2] + 80 * dt
		elseif down 'n'		then sz = sz - 40 * dt
		elseif down 'm'		then sz = sz + 40 * dt
		elseif down 'c'		then sp = sp - .005
		elseif down 'v'		then sp = sp + .005
		end
	end

function love.draw()
	for j = 1, #sur do
		quad = {}
		nrm  = {}
		for i = 1, #sur[j] do
			quad[#quad+1] = Ver[sur[j][i]][1]
			quad[#quad+1] = Ver[sur[j][i]][2]
			nrm[i] = { Ver[sur[j][i]][1], Ver[sur[j][i]][2], Ver[sur[j][i]][3] }
			end
		local x, y, z = surface_normal( nrm )
		local nx, ny = d3TOd2( x, y, z, {0,0,0}, {0,0,0} )
		if z > 0 then		-- don't draw back-facing facets
			gr.setColor(185*z,155*z,55*z,255)
			gr.polygon( 'fill', quad )
			if showNormals then
				gr.setColor( 155, 155, 155, 255* z^.08 )
				gr.circle( 'line', x * sz + center[1], y * sz + center[2], 3 )
				gr.line( x * sz + center[1], y * sz + center[2], 2 * nx * sz + center[1], 2 * ny * sz + center[2] )
				end
			end
		end
	gr.setColor( 255, 255, 0, 255 )
	gr.print( 'fps: '..love.timer.getFPS(), 10, 10 )
	for i=1,#text do gr.print(text[i],650,20*i) end
	end

function love.keypressed( key )
	if key == 'escape'	then love.event.quit() 	end
	if key == 'c'		then sides = sides>4 and sides - 1 or 3 createFigure(sides) end
	if key == 'v'		then sides = sides<100 and sides + 1 or 12 createFigure(sides) end
	if key == 'b'		then showNormals = not showNormals end
	end

function d3TOd2( x,y,z, rot, c)
	-- point, rotation matrix, rotation point around whitch object rotated
	local q  = {}
	local dx = x - c[1]
	local dy = y - c[2]
	local dz = z - c[3]
	local ca = cos( rot[1] )
    local sa = sin( rot[1] )
	local cb = cos( rot[2] )
    local sb = sin( rot[2] )
    local cg = cos( rot[3] )
    local sg = sin( rot[3] )

    q[1] = cg * (cb * dx - sb * (sa * dy + ca * dz)) - sg * (ca * dy - sa * dz) + c[1]
    q[2] = sg * (cb * dx - sb * (sa * dy + ca * dz)) + cg * (ca * dy - sa * dz) + c[2]
    q[3] = sb * dx + cb * (sa * dy + ca * dz) + c[3]

	return q[1], q[2], q[3]
	end

function surface_normal( s )  -- return normal from three points on surface
	local a1, a2, a3 = s[3][1] - s[1][1], s[3][2] - s[1][2], s[3][3] - s[1][3]  -- 2 vectors
	local b1, b2, b3 = s[2][1] - s[1][1], s[2][2] - s[1][2], s[2][3] - s[1][3]
	local n1, n2, n3 = a3*b2-a2 * b3, a1*b3-a3 * b1, a2*b1-a1 * b2 -- cross product
	nl = 1*sqrt( n1 * n1 + n2 * n2 + n3 * n3 )		-- normalization constant
	return  n1 / nl, n2 / nl, n3 / nl   			-- surface normal
	end

function createFigure(sides)
	ver	  = {	}	-- vertices
	for i = 1, sides do -- top
		ver[#ver+1] = { sin((i-1)*2*pi/sides), 1, cos((i-1)*2*math.pi/sides) }
		end
	for i = 1, sides do	-- bottom
		ver[#ver+1] = { sin((i-1)*2*pi/sides), -1, cos((i-1)*2*pi/sides) }
		end
	ver[#ver+1] = {0, 1,0}	-- center top
	ver[#ver+1] = {0,-1,0}	-- center bottom
	sur	= {	}	-- CW points defining each surface (triangles)
	for i = 1,sides-1 do
		sur[#sur+1] = {i,i+sides,i+1}
		sur[#sur+1] = {i+1,i+sides,i+1+sides}
		end
	sur[#sur+1] = {sides,2*sides,1}		-- close walls
	sur[#sur+1] = {1,2*sides,sides+1}
	for i=1,sides-1 do sur[#sur+1] = {2*sides+1,i,i+1} end	-- add top
	sur[#sur+1] = {2*sides+1,sides,1}
	for i=1,sides-1 do sur[#sur+1] = {2*sides+2,i+sides+1,i+sides} end	-- add bottom
	sur[#sur+1] = {2*sides+2,sides+1,2*sides}
	end

